@@ -130,8 +130,9 @@ class Endpoint(object):
     def _send_request(self, request_dict, operation_model):
         attempts = 1
         request = self.create_request(request_dict, operation_model)
+        context = request_dict['context']
         success_response, exception = self._get_response(
-            request, operation_model, attempts)
+            request, operation_model, context)
         while self._needs_retry(attempts, operation_model, request_dict,
                                 success_response, exception):
             attempts += 1
@@ -144,7 +145,7 @@ class Endpoint(object):
             request = self.create_request(
                 request_dict, operation_model)
             success_response, exception = self._get_response(
-                request, operation_model, attempts)
+                request, operation_model, context)
         if success_response is not None and \
                 'ResponseMetadata' in success_response[1]:
             # We want to share num retries, not num attempts.
@@ -156,12 +157,32 @@ class Endpoint(object):
         else:
             return success_response
 
-    def _get_response(self, request, operation_model, attempts):
+    def _get_response(self, request, operation_model, context):
         # This will return a tuple of (success_response, exception)
         # and success_response is itself a tuple of
         # (http_response, parsed_dict).
         # If an exception occurs then the success_response is None.
         # If no exception occurs then exception is None.
+        success_response, exception = self._do_get_response(
+            request, operation_model)
+        kwargs_to_emit = {
+            'response_dict': None,
+            'parsed_response': None,
+            'context': context,
+            'exception': exception,
+        }
+        if success_response is not None:
+            http_response, parsed_response = success_response
+            kwargs_to_emit['parsed_response'] = parsed_response
+            kwargs_to_emit['response_dict'] = convert_to_response_dict(
+                http_response, operation_model)
+        service_id = operation_model.service_model.service_id.hyphenize()
+        self._event_emitter.emit(
+            'response-received.%s.%s' % (
+                service_id, operation_model.name), **kwargs_to_emit)
+        return success_response, exception
+
+    def _do_get_response(self, request, operation_model):
         try:
             logger.debug("Sending http request: %s", request)
             history_recorder.record('HTTP_REQUEST', {
@@ -195,9 +216,34 @@ class Endpoint(object):
         parser = self._response_parser_factory.create_parser(protocol)
         parsed_response = parser.parse(
             response_dict, operation_model.output_shape)
+        # Do a second parsing pass to pick up on any modeled error fields
+        # NOTE: Ideally, we would push this down into the parser classes but
+        # they currently have no reference to the operation or service model
+        # The parsers should probably take the operation model instead of
+        # output shape but we can't change that now
+        if http_response.status_code >= 300:
+            self._add_modeled_error_fields(
+                response_dict, parsed_response,
+                operation_model, parser,
+            )
         history_recorder.record('PARSED_RESPONSE', parsed_response)
         return (http_response, parsed_response), None
 
+    def _add_modeled_error_fields(
+            self, response_dict, parsed_response,
+            operation_model, parser,
+    ):
+        error_code = parsed_response.get("Error", {}).get("Code")
+        if error_code is None:
+            return
+        service_model = operation_model.service_model
+        error_shape = service_model.shape_for_error_code(error_code)
+        if error_shape is None:
+            return
+        modeled_parse = parser.parse(response_dict, error_shape)
+        # TODO: avoid naming conflicts with ResponseMetadata and Error
+        parsed_response.update(modeled_parse)
+
     def _needs_retry(self, attempts, operation_model, request_dict,
                      response=None, caught_exception=None):
         service_id = operation_model.service_model.service_id.hyphenize()
@@ -232,7 +278,10 @@ class EndpointCreator(object):
                         timeout=DEFAULT_TIMEOUT,
                         max_pool_connections=MAX_POOL_CONNECTIONS,
                         http_session_cls=URLLib3Session,
-                        proxies=None):
+                        proxies=None,
+                        socket_options=None,
+                        client_cert=None,
+                        proxies_config=None):
         if not is_valid_endpoint_url(endpoint_url):
 
             raise ValueError("Invalid endpoint: %s" % endpoint_url)
@@ -246,6 +295,9 @@ class EndpointCreator(object):
             proxies=proxies,
             verify=self._get_verify_value(verify),
             max_pool_connections=max_pool_connections,
+            socket_options=socket_options,
+            client_cert=client_cert,
+            proxies_config=proxies_config
         )
 
         return Endpoint(
